"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const traverse_1 = require("./traverse");
const traversalHelper_1 = __importDefault(require("./traversalHelper"));
class CleanupHelper {
    /**
     * Cleans up any useless code.
     * @param ast The AST.
     */
    static cleanup(ast) {
        const self = this;
        (0, traverse_1.traverse)(ast, {
            enter(node, parent) {
                if (self.isEmptyVariableDeclarationStatement(node)) {
                    traversalHelper_1.default.removeNode(parent, node);
                }
            }
        });
    }
    /**
     * Returns whether a node is a variable declaration statement with no
     * declarators.
     * @param node The AST node.
     */
    static isEmptyVariableDeclarationStatement(node) {
        return (node.type == 'VariableDeclarationStatement' && node.declaration.declarators.length == 0);
    }
}
exports.default = CleanupHelper;
