"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const traverse_1 = require("./traverse");
class TraversalHelper {
    /**
     * Replaces a node within a given section of the AST.
     * @param root The root node.
     * @param node The node to replace.
     * @param replacement The replacement node, nodes or null.
     */
    static replaceNode(root, node, replacement) {
        (0, traverse_1.traverse)(root, {
            enter(n, parent) {
                if (n == node) {
                    for (const prop of Object.getOwnPropertyNames(parent)) {
                        const value = parent[prop];
                        if (value == n) {
                            if (Array.isArray(replacement)) {
                                throw new Error(`Cannot insert array of nodes into property ${prop} of ${parent.type}`);
                            }
                            parent[prop] = replacement;
                        }
                        else if (Array.isArray(value)) {
                            const array = value;
                            const index = array.indexOf(node);
                            if (index != -1) {
                                if (Array.isArray(replacement)) {
                                    array.splice(index, 1, ...replacement);
                                }
                                else if (replacement) {
                                    array[index] = replacement;
                                }
                                else {
                                    array.splice(index, 1);
                                }
                            }
                        }
                    }
                }
            }
        });
    }
    /**
     * Removes a node within the given section of the AST.
     * @param root The root node.
     * @param node The node to remove.
     */
    static removeNode(root, node) {
        this.replaceNode(root, node, null);
    }
}
exports.default = TraversalHelper;
