"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class Scope {
    /**
     * Creates a new scope.
     * @param node The node that created the scope.
     * @param parent The parent scope (optional).
     */
    constructor(node, parent) {
        this.node = node;
        this.parent = parent;
        this.children = new Map();
        this.arrays = new Map();
        if (this.parent) {
            this.parent.children.set(this.node, this);
        }
    }
    /**
     * Searches for an array by name.
     * @param name The name of the array.
     */
    findArray(name) {
        if (this.arrays.has(name)) {
            return this.arrays.get(name);
        }
        return this.parent ? this.parent.findArray(name) : null;
    }
    /**
     * Adds an array.
     * @param array The array to be added.
     */
    addArray(array) {
        this.arrays.set(array.name, array);
    }
}
exports.default = Scope;
