import Modification from '../../modification';
import * as Shift from 'shift-ast';
/**
 * This transformation recovers strings which have obfuscated via various techniques.
 *
 * Reversed strings:
 * "dlroW olleH".split("").reverse().join("") -> "Hello World"
 *
 * Strings split into char codes:
 * String.fromCharCode(72,101,108,108,111,32,87,111,114,108,100) -> "Hello World"
 */
export default class StringDecoder extends Modification {
    /**
     * Creates a new modification.
     * @param ast The AST.
     */
    constructor(ast: Shift.Script);
    /**
     * Executes the modification.
     */
    execute(): void;
    /**
     * Undoes various string operations.
     */
    private undoStringOperations;
    /**
     * Returns whether a node is an obfuscated reversed string.
     * @param node The AST node.
     * @returns Whether.
     */
    private isReversedString;
    /**
     * Returns whether a node is a string broken down into char codes.
     * @param node The AST node.
     * @returns Whether.
     */
    private isCharCodesString;
}
