"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const modification_1 = __importDefault(require("../../modification"));
const Shift = __importStar(require("shift-ast"));
const traverse_1 = require("../../helpers/traverse");
const traversalHelper_1 = __importDefault(require("../../helpers/traversalHelper"));
/**
 * This transformation recovers strings which have obfuscated via various techniques.
 *
 * Reversed strings:
 * "dlroW olleH".split("").reverse().join("") -> "Hello World"
 *
 * Strings split into char codes:
 * String.fromCharCode(72,101,108,108,111,32,87,111,114,108,100) -> "Hello World"
 */
class StringDecoder extends modification_1.default {
    /**
     * Creates a new modification.
     * @param ast The AST.
     */
    constructor(ast) {
        super('Undo string operations', ast);
    }
    /**
     * Executes the modification.
     */
    execute() {
        this.undoStringOperations();
    }
    /**
     * Undoes various string operations.
     */
    undoStringOperations() {
        const self = this;
        (0, traverse_1.traverse)(this.ast, {
            enter(node, parent) {
                if (self.isReversedString(node)) {
                    /**
                     * Handle reversed strings.
                     * "dlroW olleH".split("").reverse().join("") -> "Hello World"
                     */
                    const str = node.callee.object.callee.object.callee.object.value;
                    const reversedStr = new Shift.LiteralStringExpression({
                        value: str.split('').reverse().join('')
                    });
                    traversalHelper_1.default.replaceNode(parent, node, reversedStr);
                }
                else if (self.isCharCodesString(node)) {
                    /**
                     * Handle strings split into char codes.
                     * String.fromCharCode(72,101,108,108,111,32,87,111,114,108,100) -> "Hello World"
                     */
                    const charCodes = node.arguments.map(e => {
                        const charCode = e.value;
                        if (typeof charCode !== 'number') {
                            throw new Error('Unexpected char code type');
                        }
                        return charCode;
                    });
                    const str = new Shift.LiteralStringExpression({
                        value: String.fromCharCode.apply(undefined, charCodes)
                    });
                    traversalHelper_1.default.replaceNode(parent, node, str);
                }
            }
        });
    }
    /**
     * Returns whether a node is an obfuscated reversed string.
     * @param node The AST node.
     * @returns Whether.
     */
    isReversedString(node) {
        return (node.type === 'CallExpression' &&
            node.arguments.length === 1 &&
            node.arguments[0].type === 'LiteralStringExpression' &&
            node.arguments[0].value === '' &&
            node.callee.type === 'StaticMemberExpression' &&
            node.callee.property === 'join' &&
            node.callee.object.type === 'CallExpression' &&
            node.callee.object.arguments.length === 0 &&
            node.callee.object.callee.type === 'StaticMemberExpression' &&
            node.callee.object.callee.property === 'reverse' &&
            node.callee.object.callee.object.type === 'CallExpression' &&
            node.callee.object.callee.object.arguments.length === 1 &&
            node.callee.object.callee.object.arguments[0].type === 'LiteralStringExpression' &&
            node.callee.object.callee.object.arguments[0].value === '' &&
            node.callee.object.callee.object.callee.type === 'StaticMemberExpression' &&
            node.callee.object.callee.object.callee.property === 'split' &&
            node.callee.object.callee.object.callee.object.type === 'LiteralStringExpression');
    }
    /**
     * Returns whether a node is a string broken down into char codes.
     * @param node The AST node.
     * @returns Whether.
     */
    isCharCodesString(node) {
        return (node.type === 'CallExpression' &&
            node.arguments.length > 0 &&
            node.arguments.every(e => e.type === 'LiteralNumericExpression') &&
            node.callee.type === 'StaticMemberExpression' &&
            node.callee.property === 'fromCharCode' &&
            node.callee.object.type === 'IdentifierExpression' &&
            node.callee.object.name === 'String');
    }
}
exports.default = StringDecoder;
